//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
//function 是指定义一个函数
(function() {
    if (window.BridgeInjected) {
        return;
    }

    var messagingIframe;
    var bizMessagingIframe;
    var sendMessageQueue = [];
    var receiveMessageQueue = [];
    var messageHandlers = {};

    //这个是自定义scheme协议
    var CUSTOM_PROTOCOL_SCHEME = 'yibanjs';
    //队列消息
    var QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';

    var responseCallbacks = {};
    var uniqueId = 1;

    // 创建消息index队列iframe
    function _createQueueReadyIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
    }
    //创建消息体队列iframe
    function _createQueueReadyIframe4biz(doc) {
        bizMessagingIframe = doc.createElement('iframe');
        bizMessagingIframe.style.display = 'none';
        doc.documentElement.appendChild(bizMessagingIframe);
    }

    //set default messageHandler  初始化默认的消息线程
    function init(messageHandler) {
        if (WebViewJavascriptBridge._messageHandler) {
            throw new Error('WebViewJavascriptBridge.init called twice');
        }
        WebViewJavascriptBridge._messageHandler = messageHandler;
        var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }

    //YiBan Ext: 默认的Handler
    function _defaultHandler(data, callback) {
        if(callback) {
            callback(data)
        }
    }

    // 发送
    function send(data, responseCallback) {
        _doSend({
            data: data
        }, responseCallback);
    }

    // 注册线程 往数组里面添加值
    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }
    // 调用线程
    function callHandler(handlerName, data, responseCallback) {
        _doSend({
            handlerName: handlerName,
            data: data
        }, responseCallback);
    }

    //sendMessage add message, 触发native处理 sendMessage
    function _doSend(message, responseCallback) {
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }

        sendMessageQueue.push(message);
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
    }

    //YiBan Ext: 为了兼容JS端有两个回调，success和fail，对callHandler进行一些扩展
    function callHandlerExt(handlerName, data, successCallback, failCallback) {
        if(successCallback || failCallback) {
            var message = { handlerName: handlerName, data: data };
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = {succeed : successCallback, failed : failCallback};
            message.callbackId = callbackId;

            sendMessageQueue.push(message);
            messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
        }else {
            callHandler(handlerName, data)
        }
    }

    // 提供给native调用,该函数作用:获取sendMessageQueue返回给native,
    // 由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
    function _fetchQueue() {
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        //android can't read directly the return data, so we can reload iframe src to communicate with java
        if (messageQueueString !== '[]') {
            bizMessagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + encodeURIComponent(messageQueueString);
        }
    }

    //提供给native使用,
    function _dispatchMessageFromNative(messageJSON) {
        setTimeout(function() {
            var message = JSON.parse(messageJSON);
            var responseCallback;
            //java call finished, now need to call js callback function
            if (message.responseId) {
                responseCallback = responseCallbacks[message.responseId];
                if (!responseCallback) {
                    return;
                }
                //YiBan Ext:
                //从Native端传入的参数一定要是JSONArray格式，因为apply只接收[args]格式参数
                var callbackType = message.callbackType;
                if(typeof responseCallback == 'function') {
                    if(callbackType == 'n') {
                        var datas = JSON.parse(message.responseData);
                        responseCallback.apply(null, datas);
                    }else {
                        console.log('***** make sure invoke onCallBack() from native side *****');
                    }
                }else if(typeof responseCallback == 'object') {
                    if(callbackType) {
                        var _responseCallback;
                        if(callbackType == 's') {
                            _responseCallback = responseCallback.succeed;
                        }else if(callbackType == 'f') {
                            _responseCallback = responseCallback.failed;
                        }else {
                            console.log('***** make sure invoke onSucceed() or onFailed() from native side *****');
                        }
                        if(_responseCallback && message.responseData) {
                            var datas = JSON.parse(message.responseData);
                            _responseCallback.apply(null, datas);
                        }
                    }
                }else {
                    console.log('***** responseCallback must be "function" or "object" *****');
                }
                delete responseCallbacks[message.responseId];
            } else {
                //直接发送
                if (message.callbackId) {
                    var callbackResponseId = message.callbackId;
                    responseCallback = function(responseData) {
                        _doSend({
                            responseId: callbackResponseId,
                            responseData: responseData
                        });
                    };
                }

                var handler = WebViewJavascriptBridge._messageHandler;
                if (message.handlerName) {
                    handler = messageHandlers[message.handlerName];
                }
                //查找指定handler
                try {
                    handler(message.data, responseCallback);
                } catch (exception) {
                    if (typeof console != 'undefined') {
                        console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
                    }
                }
            }
        });
    }

    //提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null
    function _handleMessageFromNative(messageJSON) {
        console.log(messageJSON);
        if (receiveMessageQueue) {
            //添加到队列中
            receiveMessageQueue.push(messageJSON);
        }
        _dispatchMessageFromNative(messageJSON);

    }

    var WebViewJavascriptBridge = window.WebViewJavascriptBridge = {
        init: init,
        send: send,
        registerHandler: registerHandler,
        callHandler: callHandler,
        callHandlerExt: callHandlerExt,
        _fetchQueue: _fetchQueue,
        _handleMessageFromNative: _handleMessageFromNative
    };

    //下面这段代码实现JS和Native交互的通讯机制
    var doc = document;
    _createQueueReadyIframe(doc);
    _createQueueReadyIframe4biz(doc);

    //初始化设置默认Handler
    init(_defaultHandler);

    //JS Bridge注册完成标志
    window.BridgeInjected = {};
    //JS Bridge注册完成通知Native端
    callHandler('BridgeInjected', "Bridge Is Injected", null);

    console.log('***** Bridge Is Injected *****');
})();
